module('pge.utils', package.seeall)

local BGCOLOR = pge.gfx.createcolor(51, 68, 85)

local function background()
   if type(_M.background) == 'function' then
      _M.background()
   else
      if _M.background.draweasy then
         _M.background:draweasy(
            240 - _M.background:width () / 2,
            136 - _M.background:height() / 2)
      else
         pge.gfx.clearscreen(_M.background or BGCOLOR)
      end
   end
end

local DIALOG = {
   MESSAGE = {
      init = pge.utils.msginit,
      update = pge.utils.msgupdate,
      busy = PGE_UTILS_DIALOG_RUNNING,
      rsltmap = {
         [PGE_UTILS_MSG_DIALOG_RESULT_YES]  = true,
         [PGE_UTILS_MSG_DIALOG_RESULT_NO]   = false
      },
   },
   ERROR = {
      init = pge.utils.errmsginit,
      update = pge.utils.msgupdate,
      busy = PGE_UTILS_DIALOG_RUNNING,
   },
   NETWORK = {
      init = pge.utils.netinit,
      update = pge.utils.netupdate,
      busy = PGE_UTILS_DIALOG_RUNNING,
      rslt = pge.net.isconnected,
   },
}

local function showdialog(dlg, ...)
   if dlg.init(...) then
      local rslt

      while pge.running() do
         pge.gfx.startdrawing()
         background()
         pge.gfx.enddrawing()
         rslt = {dlg.update()}
         pge.gfx.swapbuffers()

         if rslt[1] ~= dlg.busy then break end
      end

      if type(dlg.rslt) == 'function' then
         return dlg.rslt()
      elseif dlg.rslt then
         return dlg.rslt
      elseif dlg.rsltmap then
         return dlg.rsltmap[rslt[1]]
      else
         return unpack(rslt)
      end
   end
end

function _M.msgbox(msg, def)
   local opt = 0
   if type(msg) == "number" then
      pge.utils.errormsginit(msg)
   else
      if type(def) == "boolean" then
         opt = opt + PGE_UTILS_MSG_DIALOG_YESNO_BUTTONS
         if not def then
            opt = opt + PGE_UTILS_MSG_DIALOG_DEFAULT_BUTTON_NO
         end
      end
   end

   return showdialog(DIALOG.MESSAGE, msg, opt)
end

function _M.netconnect()
   return showdialog(DIALOG.NETWORK)
end
