-- A template for a Game that uses state switching.
-- Example:
--    g = StateMachine()
--
--    splash = g:addState('splash')
--    menu = g:addState('menu')
--    maingame = g:addState('maingame')
--
--    g:pushState('splash')
StateMachine = class(Game)

function StateMachine:__init(states)
   Game.__init(self)

   self.states = {}
   self.stack = {}

   if type(states) == 'table' then
      self.states = table.copy(states)
   end
end

function StateMachine:addState(state)
   if classof(state) == string then
      state = State(state)
   elseif classof(state) ~= State then
      error('addState(): string or State expected')
   end
   state.game = self
   self.states[state.name] = state
   return state
end

function StateMachine:pushState(state)
   local s = self.states[state]
   if s then
      printf('starting "%s"', s.name)
      s:starting()
      table.insert(self.stack, s)
   else
      errorf('invalid state "%s"', state)
   end
end

function StateMachine:popState()
   local s = table.remove(self.stack)
   if s then
      printf('stopping "%s"', s.name)
      s:stopping()
      return s
   end
end

function StateMachine:setState(state)
   self:popState()
   self:pushState(state)
end

function StateMachine:getState()
   return self.stack[#self.stack]
end

function StateMachine:isRunning()
   return pge.running() and #self.stack > 0
end

function StateMachine:update(delta)
   local s = self:getState()
   if s then s:update(delta) end
end

function StateMachine:render()
   for i,s in ipairs(self.stack) do
      s:render()
   end
end

function StateMachine:run(state)
   if state then self:pushState(state) end
   Game.run(self)
end
