-- A template for a Game that runs many tasks concurrently.
-- Example:
--    g = TaskManager()
--
--    task1 = Task(
--
--    g:pushState('splash')
TaskManager = class(Game)

function TaskManager:__init()
   Game.__init(self)

   self.tasks = {}
end

function TaskManager:startTask(task, prio)
   task.co = coroutine.create(task.run)
   task.mgr = self
   task.prio = prio
   table.insert(self.tasks, task)
end

function TaskManager:setFocus(t)
   if #self.tasks == 0 then return end
   local p = self.tasks[1].prio
   if p and ((not t.prio) or (t.prio and (t.prio < p))) then return end

   local j = 1

   for i, task in ipairs(self.tasks) do
      if i > j then
         if task.prio then
            if t.prio and (task.prio > t.prio) then
               j = i
            end
         elseif t.prio then
            j = j
         end
      end
      if task == t then
         table.remove(self.tasks, i)
         table.insert(self.tasks, 1, task)
         return true
      end
   end
end

function TaskManager:update(delta)
   for i,task in ipairs(self.tasks) do
      local ok, msg = task.co:resume(task, delta)
      if not ok then print(msg) end
      if task.co:status() == 'dead' then
         task.co = nil
         task.mgr = nil
         table.remove(self.tasks, i)
      end
   end
end

function TaskManager:render()
   for i = #self.tasks, 1, -1 do
      self.tasks[i]:render()
   end
end
