do

local sqrt  = pge.math.sqrt
local sin   = pge.math.sin
local cos   = pge.math.cos
local atan2 = pge.math.atan2
local acos  = pge.math.acos

Vector = class()

function Vector:__init(x, y, z, polar)
   self.__type = 'Vector'
   if polar then
      local r = tonumber(x) or 0
      local p = tonumber(y) or 0
      local t = tonumber(z) or PI/2
      self.x = r * sin(t) * cos(p)
      self.y = r * sin(t) * sin(p)
      self.z = r * cos(t)
   else
      self.x = tonumber(x) or 0
      self.y = tonumber(y) or 0
      self.z = tonumber(z) or 0
   end
end

-- Standard axes
Vector.X = Vector(1,0,0)
Vector.Y = Vector(0,1,0)
Vector.Z = Vector(0,0,1)

-- Return a string representing this Vector
function Vector.__tostring(u)
   return ('<%g,%g,%g>'):format(u.x, u.y, u.z)
end

-- Return Vector with negated components
function Vector.__unm(u)
   return Vector(-u.x, -u.y, -u.z)
end

-- Add Vector components
function Vector.__add(u, v)
   if classof(u) ~= Vector or classof(v) ~= Vector then
      error('cannot add ' .. typeof(v) .. ' to ' .. typeof(u))
   end
   return Vector(u.x + v.x, u.y + v.y, u.z + v.z)
end

-- Subtract Vector components
function Vector.__sub(u, v)
   if classof(u) ~= Vector or classof(v) ~= Vector then
      error('cannot subtract ' .. typeof(v) .. ' from ' .. typeof(u))
   end
   return Vector(u.x - v.x, u.y - v.y, u.z - v.z)
end

-- Vector*number: returns scaled vector
-- Vector*Vector: returns dot product
function Vector.__mul(u, v)
   if classof(u) ~= Vector then u, v = v, u end
   if classof(v) == Vector then
      return (u.x * v.x) + (u.y * v.y) + (u.z * v.z)
   else
      v = tonumber(v)
      return Vector(u.x * v, u.y * v, u.z * v)
   end
end

-- Vector/number: returns scaled vector
-- Vector/Vector: returns cross product
function Vector.__div(u, v)
   if classof(u) ~= Vector then
      error('cannot divide by a Vector')
   end
   if classof(v) == Vector then
      return Vector(
         u.y * v.z - u.z * v.y,
         u.z * v.x - u.x * v.z,
         u.x * v.y - u.y * v.x)
   else
      return u * (1/v)
   end
end

function Vector.__eq(u, v)
   return (u.x == v.x) and (u.y == v.y) and (u.z == v.z)
end

-- Return Vector's length
function Vector:length()
   return sqrt(self * self)
end

-- Return Vector's inclination (angle from Z axis)
function Vector:incline()
   return acos(self.z / self:length())
end

-- Return Vector's azimuth (angle on X-Y plane)
function Vector:azimuth()
   if self.x == 0 and self.y == 0 then
      return 0
   else
      return atan2(self.y, self.x)
   end
end

-- Return normalized Vector
function Vector:norm()
   return self / self:length()
end

end