--------------------------------------------------------------------------------
-- string extensions

function string.gfind(str, pattern, plain)
   local init = 1
   return function()
      local r = {str:find(pattern, init, plain)}
      if r[1] then
         init = r[2] + 1
         return unpack(r)
      end
   end
end

function string.split(str, delim, max, plain)
   local rv = {}
   local init, a, b = 1, str:find(delim, 1, plain)

   max = tonumber(max) or #str

   while a and (#rv < max - 1) do
      rv[#rv+1] = str:sub(init, a - 1)
      init = b + 1
      a, b = str:find(delim, init, plain)
   end
   if init < #str then
      rv[#rv+1] = str:sub(init)
   end
   return rv
end

function string.chunk(str, size)
   local rv = {}
   for i = 1, #str, size do
      rv[#rv+1] = str:sub(i,i+size-1)
   end
   return rv
end

function string.rstrip(str, chars)
   chars = chars or '%s%z'
   return str:gsub('['..chars..']+$', '')
end

function string.lstrip(str, chars)
   chars = chars or '%s%z'
   return str:gsub('^['..chars..']+', '')
end

function string.strip(str, chars)
   return str:lstrip(chars):rstrip(chars)
end

function string.asciz(str)
   return str:gsub('%z.+$', '')
end


-- TODO: Rewrite to allow Python-style keyed formatting,
-- e.g. string.format('%(1)#04x (%(1)d)', 42) should return '0x2a (42)'
do
   --~ local string_format = string.format

   --~ function string.format(s, ...)
      --~ local t = {...}
      --~ if #t == 1 and type(t[1]) == 'table' then
         --~ t = t[1]
      --~ end
      --~ --return (s:gsub("%%%((.-)%)([-+#0 ]-[0-9]*.?[0-9]*[cdEefGgioqsuX])",
      --~ --   function(k, f) print(k,f) return string_format(f, t[k] or t[tonumber(k)]) end))
      --~ return string_format(s, unpack(t))
   --~ end
end

--print(string.format('%(test)#04x', { test = 42 }))

do
   local meta = getmetatable('')

   --[[
   function meta.__index(str, key)
      if type(key) == 'number' then
         return string.sub(str, key, key)
      else
         return string[key]
      end
   end
   ]]

   function meta.__mul(str, num)
      if type(str) ~= 'string' then str, num = num, str end
      return str:rep(num)
   end

   function meta.__div(str, div)
      if type(str) ~= 'string' then str, div = div, str end
      if type(div) == 'string' then
         return str:split(div, nil, true)
      else
         return str:chunk(div)
      end
   end

   function meta.__mod(str, tbl)
      if type(str) ~= 'string' then
         error('attempt to perform arithmetic on a string value')
      end
      return str:format(tbl)
   end

   function meta.__unm(str)
      return str:reverse()
   end
end
