-- Creates all directories in a path.
function pge.dir.mkdirs(path)
   do return pge.io.mkdir(path,true) end
   local p = ''
   for dir in path:gmatch('[/\\]?[^/\\]+[/\\]?') do
      p = p .. dir
      if dir:sub(-2) ~= ':/' then
         print('checking dir ' .. p)
         if not pge.dir.exists(p) then
            print'doesn\'t exist, creating'
            if not pge.dir.mkdir(p) then
               print'error!'
               return false
            end
         end
         print'okay'
      end
   end
   return true
end

-- Returns the filename portion of 'path'.
function pge.dir.basename(path)
   return path:sub((path:find('[/\\][^/\\]*$') + 1) or 1)
end

-- Returns the directory portion of 'path'.
function pge.dir.dirname(path)
   return path:sub(1, path:find('[/\\][^/\\]*$') or 0)
end

-- Returns a table of contents of a directory.
function pge.dir.contents(path)
   local dir = pge.dir.open(path)
   if not dir then return {} end
   local entries = dir:read() dir:close()
   return entries
end

-- Walks through the contents of a directory.
--   filter - Called for each filename. If it returns
--            'false' the filename is ignored.
--   callback - If given, called with each filename that
--              passes 'filter'. If omitted, a list of
--              results is returned.
--   recurse - If true or omitted, walks through all
--             directories below 'root', recursively.
function pge.dir.walk(root, filter, callback, recurse)
   root = root or pge.dir.getcwd()
   filter = filter or function (fn) return true end
   if recurse == nil then recurse = true end

   local rv, paths

   if type(root) == 'table' then
      paths = root
   else
      paths = { root }
   end

   if not callback then
      rv = {}
   end

   while next(paths) do
      local path = table.remove(paths, 1)

      for i,e in ipairs(pge.dir.contents(path)) do
         local p = path .. '/' .. e.name

         if e.dir then
            if recurse then
               table.insert(paths, p)
            end
         elseif filter(p) then
            if callback then
               callback(p)
            else
               table.insert(rv, p)
            end
         end
      end
   end

   if not callback then
      return rv
   end
end
