--------------------------------------------------------------------------------
-- pge.math

-- Export common functions into global namespace
abs    = pge.math.abs
ceil   = pge.math.ceil
floor  = pge.math.floor
sqrt   = pge.math.sqrt
max    = pge.math.fmax
min    = pge.math.fmin
exp    = pge.math.exp
sin    = pge.math.sin
asin   = pge.math.asin
cos    = pge.math.cos
acos   = pge.math.acos
sincos = pge.math.sincos
atan   = pge.math.atan
atan2  = pge.math.atan2

srand     = pge.math.srand
randint   = pge.math.randint
randfloat = pge.math.randfloat

function random(a, b)
   if b then
      return randint(a,b)
   elseif a then
      return randint(1,a)
   else
      return randfloat(0,1)
   end
end


-- deg/rad functions reversed are backward 0.02 and earlier
if pge.version() >= 3 then
   deg = pge.math.deg
   rad = pge.math.rad
else
   deg = pge.math.rad
   rad = pge.math.deg
end

-- Some useful and/or interesting constants
PI     = atan2(0, -1)    -- 3.14159...
TWOPI  = PI * 2          -- 6.28319...
HALFPI = PI / 2          -- 1.57080...
INVPI  = 1 / PI          -- 0.31831...
E      = exp(1)          -- 2.71828...
PHI    = (1 + sqrt(5))/2 -- 1.61803...

-- Additional helper functions

-- Get the signum (sign) of x
function sgn(x)
   if x > 0 then
      return 1
   elseif x < 0 then
      return -1
   else
      return 0
   end
end

-- Round 'n' to the nearest 's' (default=1)
function round(n,s)
   s = tonumber(s) or 1
   return s*floor((n + s*0.5)/s)
end

-- Clamp 'val' between 'lo' and 'hi'
function clamp(val, lo, hi)
   if lo >= hi or val < lo then
      return lo
   elseif val > hi then
      return hi
   else
      return val
   end
end

-- Wrap 'val' between 'lo' and 'hi'
function wrap(val, lo, hi)
   if lo >= hi then
      return lo
   else
      return lo + (val - lo) % (hi - lo)
   end
end

-- Convert string to integer (little endian)
--   s: string to convert
--   i: starting index (optional, default=1)
--   j: ending index (optional, default=#s)
--   u: if true, treat as unsigned
function s2i(s,i,j,u)
   i = tonumber(i) or 1
   j = tonumber(j) or #s
   local r,m = 0,1
   --print(i,j)
   for n,b in ipairs({string.byte(s,i,j)}) do
      --print(r,n,b)
      r = r + b * m
      m = m * 256
   end
   if not u then
      local n = 2^(8*(j-i+1)-1)
      --print(r,n)
      if r > n then r = r - n - n end
   end
   return r
end

-- Convert integer (little endian) to string
--   i: integer to convert
--   l: number of bytes to return (default=4)
--   u: if true, treat as unsigned
-- [Results undefined for l > 4]
function i2s(i,l,u)
   i = tonumber(i) or 0
   l = tonumber(l) or 4
   if not u then
      local n = 2^(8*l-1)
      if i > n then i = i + n + n end
   end
   local t,m = {},1
   for n = 1,l do
      t[n] = floor(i/m)%256
      m = m * 256
   end
   return string.char(unpack(t))
end
