-- Registry parser
-- Made with help from 'parsedreg3.c' by Skylark

local min = math.min

do
   local function parseSubdir(dreg, ofs, entry)
      --local name = dreg:sub(ofs+1,ofs+27):asciz()
      -- Do nothing...
      -- NOTE: not sure what "subdir" entries are for, they
      --       seem to contain the same dirs as the index?
   end
   local function parseInteger(dreg, ofs, entry)
      local name = dreg:sub(ofs+1,ofs+27):asciz()
      local value = s2i(dreg, ofs+28, ofs+31)
      --printf('[int] %s = %d', name, value)
      entry.values[name] = value
   end
   local function parseString(dreg, ofs, entry)
      local name = dreg:sub(ofs+1,ofs+27):asciz()
      local length = s2i(dreg, ofs+28, ofs+29, true)
      local k,l = dreg:byte(ofs+31), length
      local value = {}
      while l > 0 do
         local m = min(32, l)
         local ofs = 32*(k%16)+512*entry.fat[floor(k/16)+1]+1
         value[#value+1] = dreg:sub(ofs, ofs + m - 1)
         k = k + 1
         l = l - m
      end
      value = table.concat(value):asciz()
      --printf('[str] %s = %q (%d)', name, value, length)
      entry.values[name] = value
   end
   local function parseHeader(dreg, ofs, entry)
      -- Do nothing...
   end

   local parseFuncs = {
      parseSubdir,
      parseInteger,
      parseString,
      parseString,
      [15]=parseHeader,
      [16]=parseHeader,
      [31]=parseHeader,
      [32]=parseHeader,
      [47]=parseHeader,
      [48]=parseHeader,
      [63]=parseHeader,
      [64]=parseHeader,
      }

   local function loadRegistry()
      local dreg, ireg

      -- Load registry hive
      dreg = pge.file.readfile('flash1:/registry/system.dreg')
      -- Load registry index
      ireg = pge.file.readfile('flash1:/registry/system.ireg'):sub(93)

      if dreg and ireg then
         local index = {}

         for i = 1, 256 do
            local n = 58*(i-1)
            local e = {
               values = {},
               children = {},
               parent = s2i(ireg, n+5, n+6)+1,
               nent = s2i(ireg, n+11, n+12, true),
               nblk = s2i(ireg, n+13, n+14, true),
               name = ireg:sub(n+15, n+42):asciz(),
               fat = {
                  s2i(ireg, n+45, n+46, true),
                  s2i(ireg, n+47, n+48, true),
                  s2i(ireg, n+49, n+50, true),
                  s2i(ireg, n+51, n+52, true),
                  s2i(ireg, n+53, n+54, true),
                  s2i(ireg, n+55, n+56, true),
                  s2i(ireg, n+57, n+58, true),
               }}
            for j = 0, e.nent-1 do
               local ofs = 32*(j%16)+512*e.fat[floor(j/16)+1]+1
               local t = dreg:byte(ofs)
               local f = parseFuncs[t]
               if f then
                  f(dreg, ofs, e)
               --else
               --   printf('Unknown registry entry type: %02X', t)
               end
            end
            index[i] = e
         end
         -- Connect child and parent nodes
         for i, e in ipairs(index) do
            if e.nblk > 0 and e.parent > 0 then
               e.parent = index[e.parent]
               table.insert(e.parent.children, e)
            else
               e.parent = nil
            end
         end

         local _Registry = {}

         -- Build registry tree recursively
         local function buildTree(node, entry)
            for i, e in ipairs(entry.children) do
               local n = {}
               buildTree(n, e)
               node[e.name] = n
            end
            for k, v in pairs(entry.values) do
               node[k] = v
            end
         end
         for i, e in ipairs(index) do
            if e.nblk > 0 and not e.parent then
               local n = {}
               buildTree(n, e)
               _Registry[e.name] = n
            end
         end

         return _Registry
      end
   end
   --[[
   local function printTree(name,sub,depth)
      depth = depth or 0
      if type(sub) == 'table' then
         print('  '*depth .. name)
         local keys = {}
         for k,v in pairs(sub) do
            keys[#keys+1] = k
         end
         table.sort(keys)
         for i,k in ipairs(keys) do
            printTree(k,sub[k],depth+1)
         end
      else
         print('  '*depth .. name .. ' = ' .. sub)
      end
   end

   function printRegistry()
      for k,v in pairs(Registry) do
         if k[1] ~= '_' then
            printTree(k,v)
         end
      end
   end
   ]]

   if not PGEX_NO_REGISTRY then
      --local timer = pge.timer.create()
      --print('Loading registry...')
      Registry = loadRegistry() or error('Failed loading registry')
      --printf('Done, took %fs', timer:peekdelta())
      --printRegistry()
   end
end
