--------------------------------------------------------------------------------
-- pge.texture / pge.font

-- Automatically activate textures & fonts as needed,
-- and keep track of the currently active item.

do
   -- Use weak values to let unused textures be collected.
   local CACHE = setmetatable({}, { __mode = 'v' })
   -- The currently active texture.
   local ACTIVE = nil

   local pge_texture_load = pge.texture.load
   local pge_texture_activate = pge.texture.activate
   local pge_texture_draw = pge.texture.draw
   local pge_texture_draweasy = pge.texture.draweasy
   local pge_font_load = pge.font.load
   local pge_font_activate = pge.font.activate
   local pge_font_print = pge.font.print
   local pge_font_printcenter = pge.font.printcenter

   function pge.texture.load(fn, ...)
      if not CACHE[fn] then
         CACHE[fn] = pge_texture_load(fn, ...)
      end
      return CACHE[fn]
   end

   function pge.texture:activate()
      if ACTIVE ~= self then
         pge_texture_activate(self)
         ACTIVE = self
      end
   end

   function pge.texture:draw(...)
      self:activate()
      pge_texture_draw(self, ...)
   end

   function pge.texture:draweasy(...)
      self:activate()
      pge_texture_draweasy(self, ...)
   end

   function pge.font.load(fn, size, ram)
      key = fn .. '?' .. size
      if not CACHE[key] then
         CACHE[key] = pge_font_load(fn, size, ram or PGE_RAM)
      end
      return CACHE[key]
   end

   function pge.font:activate()
      if ACTIVE ~= self then
         pge_font_activate(self)
         ACTIVE = self
      end
   end

   function pge.font:print(...)
      self:activate()
      pge_font_print(self, ...)
   end

   function pge.font:printcenter(...)
      self:activate()
      pge_font_printcenter(self, ...)
   end
end

-- Pseudo-classes for PGE types
function Texture(t, ...)
   if type(t) == 'string' then -- load texture
      return pge.texture.load(t, ...)
   elseif type(t) == 'number' then -- new texture
      return pge.texture.create(t, ...)
   else
      return t
   end
end

Color = pge.gfx.createcolor
Font = pge.font.load
