abs   = pge.math.abs   or math.abs
min   = pge.math.min   or math.min
max   = pge.math.max   or math.max
floor = pge.math.floor or math.floor
ceil  = pge.math.ceil  or math.ceil

function round(n)
   return floor(n+0.5)
end

function clamp (val, lo, hi)
   return max(lo, min(hi, val or lo))
end

function wrap (val, lo, hi)
   if lo > hi then
      lo, hi = hi, lo
   elseif lo == hi then
      return lo
   end

   return lo + (val - lo) % (hi - lo)
end

function each(tbl)
   local key, val
   return function()
      key, val = next(tbl, key)
      return val
   end
end

function subset (tbl, first, len)
   local i, j = max(1, first), min(#tbl, first + len - 1)
   return function()
      if i <= j then
         i = i + 1
         return i - 1, tbl[i - 1]
      end
   end
end

local _tostring = tostring
function tostring(val)
   if type(val) == 'table' then
      local s = '{'
      for k, v in pairs(val) do
         s = s .. '[%s] = %s, ' % {tostring(k), tostring(v)}
      end
      return s .. '}'
   else
      return _tostring(val)
   end
end

--------------------------------------------------------------------------------

function table.find(tbl, item)
   for i, v in ipairs(tbl) do
      if v == item then return i end
   end
   return 0
end

function table.copy(tbl, copy)
   if copy == nil then copy = {} end
   for k, v in pairs(tbl) do
      copy[k] = v
   end
   return copy
end

--------------------------------------------------------------------------------

smt = getmetatable('')

function smt:__mod(rhs)
   if type(rhs) ~= 'table' then
      rhs = { rhs }
   end

   local s = self:gsub('%%%(([^%%%)]+)%)(%d*%.?%d*[cdEefGgioqsuXx])',
      function (name, fmt)
         name = tonumber(name) or name
         return ('%' .. fmt):format(rhs[name])
      end)

   return s:format(unpack(rhs))
end

function smt:__mul(rhs)
   return self:rep(tonumber(rhs))
end

function string.gfind(s, pattern, init, plain)
   return function()
      local r = {s:find(pattern, init, plain)}
      if r[2] then init = r[2] + 1 end
      return unpack(r)
   end
end

--------------------------------------------------------------------------------

offline = false

function ap_connect()
   if not offline and not pge.net.isconnected() then
      pge.net.init()
      if screen:showDialog(Dialog.network()) ~= 0 then
         offline = true
      end
   end
   return pge.net.isconnected()
end

function ap_disconnect()
   pge.net.shutdown()
   offline = false
end

--------------------------------------------------------------------------------

local _t_activate = pge.texture.activate
local _t_draw = pge.texture.draw
local _t_draweasy = pge.texture.draweasy
local _f_activate = pge.font.activate
local _f_print = pge.font.print

local _active

function pge.texture.activate(t)
   if _active_t ~= t then
      _t_activate(t)
      _active = t
   end
end

function pge.texture:draw(...)
   self:activate()
   _t_draw(self, ...)
end

function pge.texture:draweasy(...)
   self:activate()
   _t_draweasy(self, ...)
end

function pge.font.activate(f)
   if _active_f ~= f then
      _f_activate(f)
      _active = f
   end
end

function pge.font:print(...)
   self:activate()
   _f_print(self, ...)
end

local ROTATE90 = pge.math.atan2(-1, 0)

function pge.font:printvertical(x, y, color, text)
   local w, h = self:measure(text), self:height()
   local t = pge.texture.create(w, h, PGE_PIXEL_FORMAT_8888, PGE_VRAM)
   t:settarget()
   self:print(0, 0, color, text)
   pge.gfx.rendertoscreen()
   t:draweasy(x + w/2, y - w/2, ROTATE90)
end

