require "class"
require "extend"
require "pgex"


Sprite = class()

function Sprite:init(texture, x, y, w, h, cx, cy)
   self.texture = texture
   self.x = x or 0
   self.y = y or 0
   self.w = w or t:width() - self.x
   self.h = h or t:height() - self.y

   self:setcenter(cx or 0, cy or 0)
end

function Sprite:setcenter(cx, cy)
   self.cx = cx
   self.cy = cy
end

function Sprite:draw(x, y, sx, sy, rot, alpha)
   sx = sx or 1
   sy = sy or 1
   local w = self.w * sx
   local h = self.h * sy

   self.texture:draw(
      x - self.cx * sx, y - self.cy * sy, w, h,
      self.x, self.y, self.w, self.h, rot or 0, alpha or 255
      )
end



Controls = class()

Controls._buttons = {
   PGE_CTRL_SELECT   = 'select',
   PGE_CTRL_START    = 'start',
   PGE_CTRL_UP       = 'up',
   PGE_CTRL_RIGHT    = 'right',
   PGE_CTRL_DOWN     = 'down',
   PGE_CTRL_LEFT     = 'left',
   PGE_CTRL_LTRIGGER = 'L',
   PGE_CTRL_RTRIGGER = 'R',
   PGE_CTRL_TRIANGLE = 'triangle',
   PGE_CTRL_CIRCLE   = 'circle',
   PGE_CTRL_CROSS    = 'cross',
   PGE_CTRL_SQUARE   = 'square',
}

function Controls:init()
   self.pressed  = {}
   self.released = {}
   self.held     = {}

   self:update()
end

function Controls:update()
   pge.controls.update()

   for k, v in pairs(self._buttons) do
      self.pressed [v] = pge.controls.pressed (k)
      self.released[v] = pge.controls.released(k)
      self.held    [v] = pge.controls.held    (k)
   end

   self.ax = pge.controls.analogx()
   self.ay = pge.controls.analogy()
   self.ar = pge.controls.analogmag()
   self.aa = pge.controls.analogangle()
end


Widget = class()

function Widget:init(x, y, w, h)
   self:setbounds({
      x = x or 0,
      y = y or 0,
      w = w or 480 - self.x * 2,
      h = h or 272 - self.x,
      })
   self.widgets = {}
end

function Widget:interact(controls)
   for w in each(self.widgets) do
      w:interact(controls)
   end
end

function Widget:update()
   for w in each(self.widgets) do
      w:update()
   end
end

function Widget:draw()
   for w in each(self.widgets) do
      w:draw()
   end
end

function Widget:setbounds(bounds)
   self.x = bounds.x
   self.y = bounds.y
   self.w = bounds.w
   self.h = bounds.h
end

ProgressBar = class(Widget)

function ProgressBar:init(max, min, val, orient)
   Widget.init(self)
   self.max = max
   self.min = min or 0
   self.val = val or self.min
   self.orient = orient or 'h'
end

ScrollBar = class(Widget)

function ScrollBar:init(max, min, val, orient)
   Widget.init(self)
   self.max = max
   self.min = min or 1
   self.val = val or self.min
   self.orient = orient or 'v'
end

TextBox = class(Widget)

function TextBox:init(text, multiline, formatted)
   Widget.init(self)
   self.text = text
   self.multiline = multiline
   self.formatted = formatted

   self._scroll = ScrollBar(
end

ListBox = class(Widget)

function ListBox:init(items)
   Widget.init(self)
   self.items = items or {}
end

TitleBar = class(Widget)

function TitleBar:init(text)
   Widget.init(self)
   self.text = text or ''
end

StatusBar = class(Widget)

function StatusBar:init(buttons, text)
   Widget.init(self)
   self.buttons = buttons or {}
   self.text = text or ''
end

Window = class(Widget)

function Window:init(title, buttons, status, widget)
   self.tbar = TitleBar(title)
   self.sbar = StatusBar(buttons, status)
   self.ctrl = widget or Widget()
end

function Window:update()
   self.tbar.x = self.x
   self.tbar.y = self.y
   self.tbar.w = self.w
   self.tbar.h = GUI.titlefont:height() + 2

   self.sbar.w = self.w
   self.sbar.h = GUI.statusfont:height() + 2
   self.sbar.x = self.x
   self.sbar.y = self.y + - self.sbar.h
end


Screen = class(Widget)

function Screen:init()
   Widget.init(self, 0, 0, 480, 272)
end




Theme = class()

function Theme:draw_ProgressBar(w)
end

function Theme:draw_ScrollBar(w)
end

function Theme:draw_TitleBar(w)
end

function Theme:draw_StatusBar(w)
end

function Theme:draw_TextBox(w)
end

function Theme:draw_ListBox(w)
end





GUI = class()

function GUI:init()
   self.delta = 0
   self.widgets = {}
   self._focus = 0
end

function GUI:focus(w)
   if w then
      self._focus = table.find(self._widgets, w)
   else
      return self._widgets[self._focus]
   end
end

function GUI:interact()
   self.controls:update()

   if self:focus() then
      self:focus():interact(self.controls)
   end
end

function GUI:update()
   for i, w in self.widgets do
      w:update()
   end
end

function GUI:draw(dialog)
   pge.gfx.startdrawing()

   for i, w in each(self.widgets) do
      w:draw()
   end

   pge.gfx.enddrawing()

   if dialog then
      dialog:update()
   end

   pge.gfx.swapbuffers()
end

GUI = GUI()
