require "class"

local net = pge.net
local assert = assert

module("HTTP")

local HTTP = _M

local function httpRequest(method, url, data)
   assert(method == 'GET' or method == 'POST', 'unsupported HTTP method')
   local protocol, host, _, port, path, _, query, section = {('^(%w+://)?([%w%.%-]+)(:(%d+))?((/[^/:%?#]*)*)([;%?][^#]*)?(#.*)?$'):match(url)}

   assert(host, 'no host in URL: ' .. url)
   if not host then return nil end

   if not port then port = 80 end

   local ip = net.resolve(host)

   assert(host, 'error resolving host name (' .. host .. ')')
   if not ip then return nil end

   local socket = net.socket.create()

   local ok = socket:connect(ip, port)
   assert(ok, 'error connecting to host: ' .. ip .. ':' .. port)

   if not ok then return nil end

   socket:send(('%s %s HTTP/1.1\nHost: %s\n'):format(method, path..query, host))
   if method == 'POST' and type(data) == 'string' then
      socket:send(('Content-Length: %d\n\n%s'):format(data:len(), data))
   else
      socket:send('\n')
   end

   local response, chunk = ''

   repeat
      chunk = socket:receive()
      response = response .. chunk
   until chunk:len() == 0

   socket:close()

   local headers = {}
   local i = response:find('\n\n')

   local function httpHeader (name, value)
      headers[name] = value
   end

   response:sub(1, i):gsub('([%w-]+): +(.+)\n', httpHeader)

   response = response:sub(i+2)

   assert(response:len() > 0, 'zero-length response')

   return response
end

function HTTP.GET (url)
   return httpRequest('GET', url)
end

function HTTP.POST (url, data)
   return httpRequest('POST', url, data)
end
