-- Display a standard message dialog.
local function show_dialog(msg, options)
   if pge.utils.msginit(msg, options or 0) then
      local dlgrslt
      while pge.running() do
         pge.gfx.startdrawing()
         pge.gfx.clearscreen()
         pge.gfx.enddrawing()

         dlgrslt = pge.utils.msgupdate()
         if dlgrslt ~= PGE_UTILS_DIALOG_RUNNING then
            break
         end

         pge.gfx.swapbuffers()
      end
      return dlgrslt
   end
end

-- Enable USB for code editing; disables USB after dialog is closed.
local function enable_usb()
   pge.usb.init()
   pge.usb.activate()
   show_dialog('USB enabled. Close this dialog to continue.')
   pge.usb.deactivate()
end

-- Display the error message and stack traceback.
local function error_handler (msg)
   if show_dialog(
      (debug.traceback(msg, 2) or 'No traceback available.') .. '\n\nEnable USB?',
      PGE_UTILS_MSG_DIALOG_YESNO_BUTTONS
      ) == PGE_UTILS_MSG_DIALOG_RESULT_YES
   then
      enable_usb()
   end
end


local function copy_table (tbl)
   local copy = {}
   for k, v in pairs(tbl) do
      copy[k] = v
   end
   return copy
end

local function new_environment()
   return setmetatable({}, { __index = _G })
end

local modules = copy_table(package.loaded)

while pge.running() do
   local main, msg = loadfile("main.lua")

   if main then
      setfenv(main, new_environment())
      local ok, msg = xpcall(main, error_handler)

      if ok then break end
   else
      error_handler(msg)
   end

   local rslt = show_dialog(
      'Restart?',
      PGE_UTILS_MSG_DIALOG_YESNO_BUTTONS)
   if rslt ~= PGE_UTILS_MSG_DIALOG_RESULT_YES then
      break
   end

   -- Restore the table of loaded modules.
   package.loaded = copy_table(modules)

   -- Perform a garbage collection to free memory.
   collectgarbage('collect')
end
