function parse_feed_date(date)
   local d = {date:match('^(%d%d%d%d)%-(%d%d)%-(%d%d)T(%d%d)%:(%d%d)%:(%d%d%.?%d-)(.-)$')}
   if d[7] ~= 'Z' then
      local m = d[7]:match('([%-%+])(%d%d):(%d%d)')
      if m then
         local sign = (m[1] == '-' and -1 or 1)
         d[5] = d[5] + m[2] * sign
         d[6] = d[6] + m[3] * sign
      end
   end
   return os.time({ year = d[1], month  = d[2], day    = d[3],
                    hour = d[4], minute = d[5], second = d[6] })
end
--[[
if not pge.file.exists("mxml.txt") then
   local f = pge.file.open("mxml.txt","wt")
   for k, v in pairs(mxml) do
      f:write(k.."\n")
   end
   f:close()
end
]]
Person = class()

--[[
function Person:__init(node)
   local n
   n = node:find("name")
   if n then self.name = tostring(n:child())
end
]]

function Person:__init(name, uri, email)
   self.name = name
   self.uri = uri
   self.email = email
end

Entry = class()

function Entry:__init(id, title, updated)
   self.id = id
   self.title = title
   self.updated = updated
   self.link = {}
end

Feed = class()

--[[
Feed = "/atom/{etc}"
<category term="{category}"/>
<category term="{language}"/>
<category term="Fiction Rated: {rating}"/>
]]

function Feed:__init(file, url, overwrite)
   self.file = file
   self.url = url
   self.entry = {}
   self.link = {}

   self:load(overwrite)
end

function Feed:load(overwrite)
   local xml = getfile(self.file, self.url, overwrite)

   if xml then
      local feed = XML(xml).feed

      if not feed then
         xml = getfile(self.file, self.url, true)
         feed = XML(xml).feed
         if not feed then return end
      end

      self.id = feed.id['$']
      self.title = feed.title['$']

      self.link = {}

      local links = (#feed.link == 0) and { feed.link } or feed.link
      for i, link in ipairs(links) do
         self.link[link['@rel']] = link['@href']
      end

      self.url = self.link.self or self.url

      self.entry = {}

      local entries = (#feed.entry == 0) and { feed.entry } or feed.entry
      for i, entry in ipairs(entries) do
         --~ print(entry.id, entry.title, entry.updated)

         if entry.id and entry.title and entry.updated then

         local ev = Entry(entry.id['$'], entry.title['$'], entry.updated['$'])

         table.insert(self.entry, ev)

         if entry.author then
            local name, uri, email
            name  = entry.author.name  and entry.author.name ['$']
            uri   = entry.author.uri   and entry.author.uri  ['$']
            email = entry.author.email and entry.author.email['$']

            ev.author = Person(name, uri, email)
         end

         local links = (#entry.link == 0) and { entry.link } or entry.link
         for i, link in ipairs(links) do
            ev.link[link['@rel']] = link['@href']
         end

         ev.summary = entry.summary['$']

         end
      end

      db.feed[self.file] = {
         title = (self.title or ""):gsub('^FanFiction%.Net %- ', ''),
         url = self.url,
         }
   end
end








