local tonumber = tonumber
local tostring = tostring
local print = print
local dofile = dofile

module("FFN")
local FFN = _M

--[[ FFN constants ]]

FFN.home = "http://www.fanfiction.net/"
--FFN.home = "http://m.fanfiction.net/" -- mobile site (doesn't work with current regexes)
--FFN.home = "http://fanfiction.ath.cx:8888/" -- local dummy site
FFN.atom = "http://www.fanfiction.net/atom/"

FFN.db = "ms0:/FFNDB/"
FFN.feeddir = FFN.db .. "feed/"

--[[ FFN helper functions ]]

function FFN.urlEscape(s)
    return (s:gsub('[^-_.%w]', function(c)return('%%%02X'):format(c:byte())end))
end

function FFN.justIn(category, content, language)
   category = tonumber(category) or 0
   content  = tonumber(content)  or 0
   language = tonumber(language) or 0

   if not category then return nil end

   return (FFN.feeddir .. "justin/%08d.xml"):format(category),
      FFN.atom .. ("j/%d/%d/%d/"):format(category, content, language)
end

function FFN.list(category, rating, genre, language, sort, length, status)
   category = tonumber(category)
   rating   = tonumber(rating)   or 3
   genre    = tonumber(genre)    or 0
   language = tonumber(language) or 0
   sort     = tonumber(sort)     or 1
   length   = tonumber(length)   or 0
   status   = tonumber(status)   or 0

   if not category then return nil end

   return (FFN.feeddir .. "list/%08d.xml"):format(category),
      FFN.atom .. ("l/%d/%d/%d/%d/%d/%d/0/0/%d/0/1/"):format(
         category, rating, genre, language, sort, length, status)

end

function FFN.author(uid)
   uid = tonumber(uid)

   if not uid then return nil end

   return (FFN.feeddir .. "user/%08d.xml"):format(uid), FFN.atom .. ("u/%d/"):format(uid)
end

function FFN.chapter(sid, num)
   sid = tonumber(sid)
   num = tonumber(num)

   if not (sid and num) then return nil end

   return (FFN.db .. "story/%08d/%08d"):format(sid, num),
      FFN.home .. ("s/%d/%d/"):format(sid, num)
end

function FFN.search(t)
    return FFN.home .. ('search.php?ready=1&type=%s&plus_keywords=%s&minus_keywords=%s&categoryid=%d'):
        format(t.type or 'story', FFN.urlEscape(t.plus_keywords or ''), FFN.urlEscape(t.minus_keywords or ''), t.categoryid)
end

function FFN.catname(id)
    return FFN.category[id] and FFN.category[id].name or '?'
end

function FFN.searchResults(s)
    if not s then return end

    --~ <a +href="/s/(%d+)/1/.-">(.-)</a> +by +<a href="/u/80402/.-">(.-)</a>
    --~ <a +href="/s/(%d+)/1/.-">(.-)</a> *<a href="/s/%d+/(%d+)/.-">.-</a> +by +<a href="/u/(%d+)/.-">(.-)</a>
    local r = {}
    local s1, s2, s3 =
        -- multiple chapters
        '<a +href="/s/(%d+)/1/.-">(.-)</a> *<a href="/s/%1/(%d+)/.-"> *&#187; *</a> +by +<a href="/u/(%d+)/.-">(.-)</a>',
        -- one chapter
        '<a +href="/s/(%d+)/1/.-">(.-)</a> +by +<a href="/u/(%d+)/.-">(.-)</a>',
        -- author search
        "<a +href='/u/(%d+)/.-'>(.-)</a> *:(%d+)"

    local i, j, k, j1, k1, j2, k2 =  1
    local sid1, title1, chapters1, aid1, author1
    local sid2, title2, aid2, author2
    local aid, author, stories

    --~ print'begin search'

    while true do
        --~ print('i =', i)

        --~ print(1)
        j1, k1, sid1, title1, chapters1, aid1, author1 = s:find(s1, i)
        j2, k2, sid2, title2, aid2, author2 = s:find(s2, i)
        j, k, aid, author, stories = s:find(s3, i)

        if j1 and k1 and (not (j2 and k2) or ((j2 <= k1) and (k2 >= j1))) then
            print'found multi-chapter story'
            print(sid1, '"'..title1..'"', chapters1, aid1, author1)

            i = k1 + 1

            r[#r+1] = {
                sid = tonumber(sid1), title = title1:gsub('%b<>',''),
                chapters = tonumber(chapters1),
                aid = tonumber(aid1), author = author2 }

        elseif j2 and k2 then
            print'found single-chapter story'
            print(sid2, '"'..title2..'"', aid2, author2)

            i = k2 + 1
            r[#r+1] = {
                sid = tonumber(sid2), title = title2:gsub('%b<>',''),
                chapters = 1, aid = tonumber(aid2), author = author2 }

        elseif j and k then
            print'found author'
            print(aid, author)

            i = k + 1
            r[#r+1] = { aid = tonumber(aid), author = author:gsub('%b<>',''), stories = stories }

        else
            --~ print'nothing found'
            break
        end
    end

    --~ print'end search'

    return r
end

function FFN.userid(uri)
   return uri and tonumber(uri:match('/u/(%d+)/'))
end

function FFN.storyid(uri)
   return uri and tonumber(uri:match('/s/(%d+)/'))
end

function FFN.nchapters(summary)
   return summary and tonumber(summary:match('Chapters: (%d+)'))
end

--[[ FFN metadata ]]

FFN.content = {
   [1]="New Stories",
   [2]="New Chapters"}
FFN.sort = {
   [1]="Update Date",
   [2]="Publish Date"}
FFN.status = {
   [1]="In-Progress",
   [2]="Complete"}
FFN.language = {
   [1]="English",
   [2]="Spanish",
   [3]="French",
   [4]="German",
   [5]="Chinese",
   [6]="Japanese",
   [7]="Dutch",
   [8]="Portuguese",
   [9]="Scandinavian",
   [10]="Russian",
   [11]="Italian",
   [12]="Bulgarian",
   [13]="Polish",
   [14]="Hungarian",
   [15]="Hebrew",
   [16]="Arabic",
   [17]="Swedish",
   [18]="Norwegian",
   [19]="Danish",
   [20]="Finnish",
   [21]="Filipino",
   [22]="Esperanto",
   [23]="Hindi",
   [24]="Punjabi",
   [25]="Farsi",
   [26]="Greek",
   [27]="Romanian",
   [28]="Albanian",
   [29]="Serbian",
   [30]="Turkish",
   [31]="Czech",
   [32]="Indonesian",
   [33]="Croatian",
   [34]="Catalan",
   [35]="Latin",
   [36]="Korean"}
FFN.genre = {
   [1]="General",
   [2]="Romance",
   [3]="Humor",
   [4]="Drama",
   [5]="Poetry",
   [6]="Adventure",
   [7]="Mystery",
   [8]="Horror",
   [9]="Parody",
   [10]="Angst",
   [11]="Supernatural",
   [12]="Suspense",
   [13]="Sci-Fi",
   [14]="Fantasy",
   [15]="Spiritual",
   [16]="Tragedy",
   [17]="Western",
   [18]="Crime",
   [19]="Family",
   [20]="Hurt/Comfort",
   [21]="Friendship"}
FFN.rating = {
   [1]="K",
   [2]="K -> K+",
   [3]="K -> T",
   [12]="K+",
   [13]="T",
   [14]="M"}
