local DEADZONE = 40
local SPEED = 0.5

local max = pge.math.max

local BUTTONS = {
   PGE_CTRL_CANCEL, PGE_CTRL_ACCEPT, PGE_CTRL_SQUARE, PGE_CTRL_TRIANGLE,
   PGE_CTRL_LTRIGGER, PGE_CTRL_RTRIGGER, PGE_CTRL_LTRIGGER+PGE_CTRL_RTRIGGER,
   PGE_CTRL_START, PGE_CTRL_SELECT
}
local BUTTONTEXT = {
   [PGE_CTRL_CROSS] = 'X',
   [PGE_CTRL_CIRCLE] = 'O',
   [PGE_CTRL_SQUARE] = 'Q', --'[]',
   [PGE_CTRL_TRIANGLE] = 'T', --'/\\',
   [PGE_CTRL_LTRIGGER] = 'L',
   [PGE_CTRL_RTRIGGER] = 'R',
   [PGE_CTRL_LTRIGGER+PGE_CTRL_RTRIGGER] = 'L/R',
   [PGE_CTRL_START] = 'S', --'St',
   [PGE_CTRL_SELECT] = 'E', --'Se',
}
local BUTTONPICS = CustomFont('res/buttons.png', {
   [0x54] = { 0,  0, 16, 16}, -- 'T' triangle
   [0x4F] = {16,  0, 16, 16}, -- 'O' circle
   [0x58] = {32,  0, 16, 16}, -- 'X' cross
   [0x51] = {48,  0, 16, 16}, -- 'Q' square
   [0x4C] = {64,  0, 16, 16}, -- 'L' L trigger
   [0x52] = {80,  0, 16, 16}, -- 'R' R trigger
   [0x53] = { 0, 32, 48, 16}, -- 'S' start
   [0x45] = {48, 32, 48, 16}, -- 'E' select
   })

--------------------------------------------------------------------------------
--[[
Dialog = class()

function Dialog:__init(dlgInit, dlgUpdate, dlgBusy)
   self.dlgInit = dlgInit
   self.dlgUpdate = dlgUpdate
   self.dlgBusy = dlgBusy
   self.dlgRslt = {}

   self.running = true
end

function Dialog:__call(...)
   self.running = self.dlgInit(...)
   return self
end

function Dialog:update()
   if self.running then
      --error("dialog")
      self.dlgRslt = {self.dlgUpdate()}
      if (not pge.running()) or (self.dlgRslt[1] ~= self.dlgBusy) then
         self.running = false
      end
   end
end

Dialog.osk = Dialog(pge.utils.oskinit, pge.utils.oskupdate, false)
Dialog.message = Dialog(pge.utils.msginit, pge.utils.msgupdate, PGE_UTILS_DIALOG_RUNNING)
Dialog.error = Dialog(pge.utils.errormsginit, pge.utils.msgupdate, PGE_UTILS_DIALOG_RUNNING)
Dialog.network = Dialog(pge.utils.netinit, pge.utils.netupdate, PGE_UTILS_DIALOG_RUNNING)
Dialog.browser = Dialog(pge.utils.browserinit, pge.utils.browserupdate, PGE_UTILS_DIALOG_RUNNING)
Dialog.save = Dialog(pge.utils.saveinit, pge.utils.savedataupdate, PGE_UTILS_DIALOG_RUNNING)
Dialog.load = Dialog(pge.utils.loadinit, pge.utils.savedataupdate, PGE_UTILS_DIALOG_RUNNING)
Dialog.autosave = Dialog(pge.utils.autosaveinit, pge.utils.savedataupdate, PGE_UTILS_DIALOG_RUNNING)
Dialog.autoload = Dialog(pge.utils.autoloadinit, pge.utils.savedataupdate, PGE_UTILS_DIALOG_RUNNING)
]]
--------------------------------------------------------------------------------

Screen = class()

function Screen:__init(backcolor, graycolor)
   self.backcolor = backcolor or Color(0, 0, 0)
   self.graycolor = graycolor or Color(0, 0, 0, 128)
   self.views = {}
   self.timer = pge.timer.create()

   self:refresh()

   --self:setmode(orient or "h")
end

--[[
function Screen:setmode(orient)
   self.orient = orient
   if self.orient == "v" then
      self.w = 256
      self.h = 480
   else
      self.w = 480
      self.h = 272
   end

   self.cx = (480 - self.w) / 2
   self.cy = (272 - self.h) / 2

   if self.texture then self.texture:toram() end
   self.texture = pge.texture.create(self.w, self.h, PGE_PIXEL_FORMAT_8888, PGE_VRAM)
   assert(self.texture, 'unable to allocate buffer')
end
]]
function Screen:view()
   return self.views[#self.views]
end

function Screen:push_view(view)
   printf('Push "%s"', view.title.text)
   table.insert(self.views, view)
   self:refresh()
end

function Screen:pop_view()
   local view = table.remove(self.views)
   if view then
       view:on_close()
       printf('Pop "%s"', view.title.text)
   end
   self:refresh()
   return view
end

function Screen:set_view(view)
   self.views[#self.views] = view
   self:refresh()
end

function Screen:interact(view, dialog)
   pge.controls.update()

   if dialog then return true end

   --~ if pge.controls.pressed(PGE_CTRL_SELECT) then
      --~ screenshot()
   --~ end

   return view:interact()
end

function Screen:update(view, dialog)
   self.timer:update()
   local delta = self.timer:getdelta()
   view:update(delta, dialog)
end

local ROTATE90 = pge.math.atan2(-1, 0)

function Screen:refresh()
    --~ print'requesting refresh'
    --~ local view = self:view()
    --~ if view then print('view title =',view.title.text) end
    self.dirty = true
end

function Screen:dim(dim)
    self.dimmed = dim

    local view = self:view()

    if view then
        self:update(view)
        self:draw(view)
    end
end

function Screen:draw(view, dialog)
    if dialog or self.dirty then
       --~ if self.dirty then
          --~ print'refreshing screen'
          --~ if view then print('view title =',view.title.text) end
       --~ end
       pge.gfx.startdrawing()

       --self.texture:settarget()

       pge.gfx.clearscreen(self.backcolor)
       view:draw()

       --pge.gfx.rendertoscreen()
       --pge.gfx.clearscreen()
       --self.texture:activate()
       --if self.orient == 'v' then
       --   self.texture:draweasy(self.cx, self.cy, ROTATE90)
       --else
       --   self.texture:draweasy(0, 0, 0)
       --end

       if dialog or self.dimmed then
          pge.gfx.drawrect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, self.graycolor)
       end

       pge.gfx.enddrawing()
       if dialog then dialog:update() end
       pge.gfx.swapbuffers()

       self.dirty = false
   else
      plug.graphics.waitVsync()
   end
end

function Screen:doFrame(dialog)
   local view

   view = self:view()
   if view then self:interact(view, dialog) end

   view = self:view()
   if view then self:update(view, dialog) end

   view = self:view()
   if view then
      self:draw(view, dialog)
      return true
   end
end

function Screen:showDialog(dialog)
   if dialog then
      while dialog:running() and self:running() do
         self:doFrame(dialog)
      end

      screen:refresh()

      return dialog:result()
   end
end

function Screen:running()
   return pge.running() and self:view()
end

function Screen:run()
   print'Screen:run()'

   while self:running() do
      self:doFrame()
   end

   while self:view() do
      self:view():close()
   end
end

screen = Screen()

--------------------------------------------------------------------------------

THEME_FORECOLOR = 'forecolor'
THEME_BACKCOLOR = 'backcolor'
THEME_INFOFORE  = 'infofore'
THEME_INFOBACK  = 'infoback'
THEME_HIGHLIGHT = 'highlight'
THEME_HOTTEXT   = 'hottext'

COLORS = {
   [00] = Color(  0,   0,   0),
   [01] = Color(  0,   0, 170),
   [02] = Color(  0, 170,   0),
   [03] = Color(  0, 170, 170),
   [04] = Color(170,   0,   0),
   [05] = Color(170,   0, 170),
   [06] = Color(170,  85,   0),
   [07] = Color(170, 170, 170),
   [08] = Color( 85,  85,  85),
   [09] = Color( 85,  85, 255),
   [10] = Color( 85, 255,  85),
   [11] = Color( 85, 255, 255),
   [12] = Color(255,  85,  85),
   [13] = Color(255,  85, 255),
   [14] = Color(255, 255,  85),
   [15] = Color(255, 255, 255),
}

function Theme(name, fc, bc, fi, bi, hl, ht)
   return {
      name = name,
      forecolor = fc,
      backcolor = bc,
      infofore = fi,
      infoback = bi,
      highlight = hl,
      hottext = ht,
      }
end

function setTheme(n)
   db.theme.selected = clamp(n, 1, #db.theme)
end
function getTheme()
   return clamp(tonumber(db.theme.selected) or 1, 1, #db.theme)
end
function theme()
   return __temptheme or db.theme[getTheme()]
end

function themeColor(id)
   return COLORS[theme()[id]]
end

function setDefaultThemes()
   table.insert(db.theme, Theme("Default",                   0, 15, 15,  1,  7,  4))
   table.insert(db.theme, Theme("Monochrome Light",          0, 15, 15,  0,  7,  4))
   table.insert(db.theme, Theme("Monochrome Dark",          15,  8, 15,  0,  0, 12))
   table.insert(db.theme, Theme("Mint & Lime",               0,  2,  0, 10, 10, 15))
   table.insert(db.theme, Theme("Silky Doll",                0, 12, 15,  4,  4, 15))
   table.insert(db.theme, Theme("Lina Inverse",              7,  0,  0, 15,  8, 15))
   table.insert(db.theme, Theme("Darkness Beyond Twilight",  7,  0,  0,  4,  4, 15))
   table.insert(db.theme, Theme("Hylian Hero",               0, 10, 14,  6, 14,  4))
   table.insert(db.theme, Theme("Red, White & Bluejeans",   15,  8, 15,  1,  4, 14))
   table.insert(db.theme, Theme("Polar Ice",                 0, 11, 15,  3,  3,  5))
   table.insert(db.theme, Theme("Night Vision",              7,  0,  0,  8,  8, 15))
   setTheme(1)
end

--------------------------------------------------------------------------------

Widget = class()

function Widget:__init(x, y, w, h)
   self.x = x
   self.y = y
   self.w = w
   self.h = h

   self.time = 0
end

function Widget:interact()
end

function Widget:update(delta)
   self.time = self.time + delta
end

function Widget:draw()
   pge.gfx.drawrect(self.x, self.y, self.w, self.h, themeColor(THEME_BACKCOLOR))
end

--------------------------------------------------------------------------------

ProgressBar = class(Widget)

function ProgressBar:__init(x, y, w, h, lo, hi, val)
   Widget.__init(self, x, y, w, h)
   self.lo = lo or 0
   self.hi = hi or 100
   self.val = val or self.lo
end

function ProgressBar:draw()
    local y, h = self.y, self.h

    if h > font:height() + 2 then
        h = font:height() + 2
        y = self.y + (self.h - h) / 2
    end

    local f = (self.val - self.lo) / max(1, self.hi - self.lo)
    local pw = (self.w - 2) * f

    pge.gfx.drawrect(self.x, self.y, self.w, self.h, themeColor(THEME_BACKCOLOR))
    pge.gfx.drawrect(self.x, y, self.w, h, themeColor(THEME_FORECOLOR))
    pge.gfx.drawrect(self.x + 1, y + 1, pw, h - 2, themeColor(THEME_HIGHLIGHT))

    local s

    if self.show_percent then
        s = ('%d%%'):format(100*f)
    elseif self.show_fraction then
        s = ('%d/%d'):format(self.val, self.hi)
    elseif self.show_number then
        s = ('%d'):format(self.val)
    end

    if s then
        font:activate()
        local w = font:measure(s)
        if pw < w then
            font:print(self.x + 1 + pw, y + 1, themeColor(THEME_HIGHLIGHT), s)
        else
            font:print(self.x + 1 + pw - w, y + 1, themeColor(THEME_FORECOLOR), s)
        end
    end
end

--------------------------------------------------------------------------------

ScrollBar = class(Widget)

function ScrollBar:__init(x, y, w, h, lo, hi, val, step)
   Widget.__init(self, x, y, w, h)
   self.lo = tonumber(lo) or 0
   self.hi = tonumber(hi) or 100
   self.val = tonumber(val) or self.lo
   self.step = tonumber(step) or 1
end

function ScrollBar:draw()
   self.val = clamp(self.val, self.lo, self.hi)

   local range = max(1, self.hi - self.lo + 1)
   local ty = self.h * (self.val - self.lo) / range
   local th = ceil(clamp(self.h * self.step / range, 1, self.h))

   pge.gfx.drawrect(self.x, self.y, self.w, self.h, themeColor(THEME_FORECOLOR))
   pge.gfx.drawrect(self.x + 1, self.y + ty, self.w - 2, th, themeColor(THEME_HIGHLIGHT))
end

--------------------------------------------------------------------------------

--[=====[

local function wrap(f,s,w)
    local r = {}
    local j, k = 1, 0
    for i = 1, #s do
        if s:byte(i) == 32 then
            k = i
        elseif f:measure(s:sub(j, i)) > w then
            if k >= j then
                r[#r + 1] = s:sub(j, k - 1)
                j = k + 1
            else
                r[#r + 1] = s:sub(j, i - 1)
                j = i
            end
        end
    end
    if j <= #s then
        r[#r + 1] = s:sub(j)
    end
    return table.concat(r,'\n');
end

function wrap_lines(text, maxwidth)
   local f = font

   local lines, line = {}, {}
   local first, last, width = 1, 0, 0

   local l, s = 1, 1

   local bold, italic, uline = false, false, false

   local function setfont(b, i)
      bold = b ; italic = i
      if b then
         if i then f = fontbi else f = fontb end
      else
         if i then f = fonti  else f = font  end
      end
   end

   local function putline()
--[[
      local tw = 0
      for i, s in ipairs(line) do
         tw = tw + s.width
      end
      local space = (maxwidth - tw) / (#line - 1)
      local x = 0
      for i, s in ipairs(line) do
         s.x = x ; x = x + s.width + space
      end
]]
      lines[l] = line ; line = {} ; l = l + 1 ; s = 1
   end

   local function putspan(t)
      if t:len() > 0 then
         --local w = f:measure(t)
         line[s] = { text = t, font = f, uline = uline }
         s = s + 1
      end
   end
--[[
    local j, k = 1, 0
    for i = 1, #s do
        if s:byte(i) == 32 then
            k = i
        elseif f:measure(s:sub(j, i)) > w then
            if k >= j then
                r[#r + 1] = s:sub(j, k - 1)
                j = k + 1
            else
                r[#r + 1] = s:sub(j, i - 1)
                j = i
            end
        end
    end
    if j <= #s then
        r[#r + 1] = s:sub(j)
    end
]]
   print'wrapping text...'

   for n = 1, text:len() do
      local c = text:byte(n)
      if c == 32 or c == 10 then
         last = n
      elseif c == 1 then
         putspan(text:sub(first, n - 1))
         first = n + 1 ; last = first
         setfont(not bold, italic)
      elseif c == 2 then
         putspan(text:sub(first, n - 1))
         first = n + 1 ; last = first
         setfont(bold, not italic)
      elseif c == 3 then
         putspan(text:sub(first, n - 1))
         first = n + 1 ; last = first
         uline = not uline
      end
      width = width + f:measure(string.char(c))
      if c == 10 or (c ~= 32 and width > maxwidth) then
         putspan(text:sub(first, last))
         putline()
         first = last + 1
         width = f:measure(text:sub(first, n))
      end
   end
   if first <= text:len() then putspan(text:sub(first)) end
   if #line > 0 then putline() end

   print'done.'
   return lines
end
--]=====]

local function makespan(s, t)
    s = s or {}
    t = t or s.text or ''

    local w = s.font:measure(t)
    local spc, brk = t:match' ' and true, t:match'\n' and true

    return { text=t, font=s.font, width=w, uline=s.uline, space=spc, brk=brk }
end

local function parse_spans(text)
    local first, last, width = 1, 0, 0

    local spans = {}

    local inword = false
    local B, I, U, f

    local function setfont(b, i, u)
        B = b ; I = i ; U = u
        if b then
            if i then f = fontbi else f = fontb end
        else
            if i then f = fonti  else f = font  end
        end
    end

    local function putspan(t)
        spans[#spans+1] = makespan({ font = f, uline = U }, t)
    end

    setfont(false, false, false)

    for n = 1, #text do
        local c = text:byte(n)

        if c == 10 then
            if inword then
                putspan(text:sub(first, n - 1))
            end
            putspan'\n'
            first = n+1 ; last = first
            inword = false
        elseif c == 1 then
            putspan(text:sub(first, n - 1))
            first = n + 1 ; last = first
            setfont(not B, I, U)
        elseif c == 2 then
            putspan(text:sub(first, n - 1))
            first = n + 1 ; last = first
            setfont(B, not I, U)
        elseif c == 3 then
            putspan(text:sub(first, n - 1))
            first = n + 1 ; last = first
            setfont(B, I, not U)
        elseif c <= 32 then
            if inword then
                putspan(text:sub(first, n - 1))
                first = n ; last = first
            end
            inword = false
        else
            if not inword then
                putspan(text:sub(first, n - 1))
                first = n ; last = first
            end
            inword = true
        end
    end

    if first <= text:len() then
        putspan(text:sub(first))
    end

    return spans
end

-- map of word hyphenations, sorted by longest first segment
local hyphenation = {
    --~ ['hyphenation'] = { 7, 6, 2 }, -- i.e. 'hy-phen-a-tion'
}

local function hyphenate(word, font, width)
    local hyph = hyphenation[word:lower()]

    if hyph then
        for i, h in ipairs(hyph) do
            local t = word:sub(1, h) .. '-'

            if font:measure(t) <= width then
                return t, word:sub(h + 1)
            end
        end
    end
end

local function wrap_lines(text, maxwidth)
    local line, lines = {}, {}

    local x, y, h = 0, 0, 0

    local function putspan(s, t)
        local s2 = line[#line]
        s = makespan(s, t)

        --~ print(('putspan("%s"), x=%d, y=%d, width=%d, space=%s, brk=%s [freeram==%.2fkB]'):
            --~ format(s.text, x, y, s.width, tostring(s.space), tostring(s.brk), plug.freeram()/1024))

        if s2 and s.font == s2.font and s.uline == s2.uline and not (s.brk or s2.brk) then
            -- merge spans to save memory
            s2.text = s2.text .. s.text
            s2.width = s2.width + s.width
            s2.space = s2.space and s.space
            x = x + s.width
            s, s2 = s2, nil
        else
            line[#line+1] = s
            s.x = x
            x = x + s.width
        end

        if h < s.font:height() then
            h = s.font:height()
        end
    end

    local function putline()
        --~ if line[1] then
            --~ if #line > 1 then
                --~ print(line[1].x, line[1].text, '...', line[#line].x, line[#line].text, '('..tostring(#line)..')')
            --~ else
                --~ print(line[1].x, line[1].text)
            --~ end
        --~ end
        line.y, line.height = y, h
        lines[#lines+1] = line
        line = {}
        x, y, h = 0, y + h, 0

        local f = plug.freeram()
        if f < 1048576 then
            collectgarbage()
            local g = plug.freeram()
            printf('%d free (%.14gkB) -> collected = %d free (%.14gkB)', f, f/1024, g, g/1024)
        end
    end

    for i, s in ipairs(parse_spans(text)) do
        if s.brk then
            putline()
        elseif not s.space and (x + s.width > maxwidth) then
            --TODO: split word if possible, then break line
            local a, b = hyphenate(s.text, maxwidth - x)
            if b then
                putspan(s, a)
                putspan(s, b)
            elseif s.width > maxwidth then
                local i, j, t, f = 1, 0, s.text, s.font
                for j = 1, #t do
                    j = j + 1
                    if x + f:measure(t:sub(i, j)..'+') > maxwidth then
                        if j > i + 1 then
                            putspan(s, t:sub(i, j)..'+')
                            putline()
                            i = j + 1
                        else
                            putline()
                        end
                    end
                end
            else
                putline()
                putspan(s)
            end
        else
            putspan(s)
        end
    end

    if #line > 0 then putline() end

    return lines
end

TextBox = class(Widget)

function TextBox:__init(x, y, w, h, text)
   Widget.__init(self, x, y, w, h)
   self.first_line = 1
   self.position = 1

   self.leading = 3
   self.line_height = font:height() + self.leading
   self.max_lines = floor((self.h - 2) / self.line_height)

   self.scroll = ScrollBar(self.x + self.w - 8, self.y, 8, self.h, 1, 0, 1, self.max_lines)

   self:settext(text)
end

function TextBox:settext(text)
   self.lines = wrap_lines(text or "", self.w - 10)
   screen:refresh()
end

function TextBox:interact()
   if pge.controls.pressed(PGE_CTRL_UP) then
      self.position = self.position - 1
      screen:refresh()
   elseif pge.controls.pressed(PGE_CTRL_DOWN) then
      self.position = self.position + 1
      screen:refresh()
   elseif pge.controls.pressed(PGE_CTRL_LEFT) then
      self.position = self.position - self.max_lines + 1
      screen:refresh()
   elseif pge.controls.pressed(PGE_CTRL_RIGHT) then
      self.position = self.position + self.max_lines - 1
      screen:refresh()
   end

   --[[
   local dy = pge.controls.analogy()
   if dy < -DEADZONE or dy > DEADZONE then
      dy = dy - dy / abs(dy) * DEADZONE
      self.position = self.position + SPEED * (dy / (128 - DEADZONE))
   end
   ]]
end

function TextBox:update(delta)
   Widget.update(self, delta)

   self.position = clamp(self.position, 1, #self.lines - self.max_lines + 1)
   self.first_line = floor(self.position)

   self.scroll.x = self.w - self.scroll.w
   self.scroll.h = self.h
   self.scroll.val = self.position
   self.scroll.hi = #self.lines
   self.scroll:update(delta)
end

function TextBox:draw()
    pge.gfx.drawrect(self.x, self.y, self.w, self.h, themeColor(THEME_BACKCOLOR))

    local x, y, f = self.x + 1, self.y + 1 - self.line_height * (self.position % 1)

    for i, line in table.range(self.lines, self.first_line, self.first_line + self.max_lines + 1) do
        for j, span in ipairs(line) do
            --local x = self.x + span.x + 1
            local w = span.width

            if not span.space then
                if span.font ~= f then f = span.font ; f:activate() end
                f:print(span.x + x, y, themeColor(THEME_FORECOLOR), span.text)
            end
            if span.uline then
                local yy = y + f:height()
                pge.gfx.drawline(span.x + x, yy, span.x + x + w, yy, themeColor(THEME_FORECOLOR))
            end
            --~ x = x + w
        end
        y = y + self.line_height
    end

    self.scroll:draw()
end

--------------------------------------------------------------------------------

ListBox = class(Widget)

function ListBox:__init(x, y, w, h, items)
   Widget.__init(self, x, y, w, h)
   self.items = items or {}
   self.selected = 1
   self.first_line = 1
   self.position = 1

   self.leading = 3
   self.line_height = font:height() + self.leading
   self.max_lines = floor((self.h - 2) / self.line_height)

   self.scroll = ScrollBar(self.x + self.w - 8, self.y, 8, self.h, 1, 0, 1, self.max_lines)
end

function ListBox:interact()
   if pge.controls.pressed(PGE_CTRL_UP) then
      if self.selected > 1 then
         self.selected = self.selected - 1
      else
         self.selected = #self.items
      end
      screen:refresh()
   elseif pge.controls.pressed(PGE_CTRL_DOWN) then
      if self.selected < #self.items then
         self.selected = self.selected + 1
      else
         self.selected = 1
      end
      screen:refresh()
   elseif pge.controls.pressed(PGE_CTRL_LEFT) then
      if self.selected > self.first_line then
         self.selected = self.first_line
      else
         self.selected = self.selected - self.max_lines + 1
      end
      screen:refresh()
   elseif pge.controls.pressed(PGE_CTRL_RIGHT) then
      if self.selected < self.first_line + self.max_lines - 1 then
         self.selected = self.first_line + self.max_lines - 1
      else
         self.selected = self.selected + self.max_lines - 1
      end
      screen:refresh()
   end

   --[[
   local dy = pge.controls.analogy()
   if dy < -DEADZONE or dy > DEADZONE then
      dy = dy - dy / abs(dy) * DEADZONE
      self.position = self.position + SPEED * (dy / (128 - DEADZONE))
      self.selected = clamp(self.selected, round(self.position), round(self.position + self.max_lines - 1))
   else]]
      self.position = clamp(self.position, self.selected - self.max_lines + 1, self.selected)
   --end
end

function ListBox:update(delta)
   Widget.update(self, delta)

   self.selected = clamp(self.selected, 1, #self.items)
   self.position = clamp(self.position, 1, #self.items - self.max_lines + 1)

   self.first_line = floor(self.position)

   self.scroll.x = self.w - self.scroll.w
   self.scroll.h = self.h
   self.scroll.val = self.position
   self.scroll.hi = #self.items
   self.scroll:update(delta)
end

function ListBox:draw()
   pge.gfx.drawrect(self.x, self.y, self.w, self.h, themeColor(THEME_BACKCOLOR))

   font:activate()

   local y = self.y + 1 - self.line_height * (self.position % 1)
   for i, item in table.range(self.items, self.first_line, self.first_line + self.max_lines + 1) do
      if i == self.selected then
         pge.gfx.drawrect(self.x, y, self.w, font:height() + self.leading, themeColor(THEME_HIGHLIGHT))
      end

      local hot = false
      item = tostring(item)
      if item:sub(1,2) == '@@' then hot = true ; item = item:sub(3) end
      if self.numbered then item = ('%d. %s'):format(i, item) end

      font:print(self.x + 1, y, themeColor(hot and THEME_HOTTEXT or THEME_FORECOLOR), item)
      y = y + font:height() + self.leading
   end

   self.scroll:draw()
end

--------------------------------------------------------------------------------

TitleBar = class(Widget)

function TitleBar:__init(x, y, w, h, text)
   Widget.__init(self, x, y, w, h)
   self.text = tostring(text or "")
end

--~ function TitleBar:update(delta)
   --~ Widget.update(self, delta)
--~ end

function TitleBar:draw()
   pge.gfx.drawrect(self.x, self.y, self.w, self.h, themeColor(THEME_INFOBACK))

   fontb:activate()
   fontb:print(self.x + 1, self.y, themeColor(THEME_INFOFORE), self.text)
   --local batt = pge.system.battery.percent()
   --fontb:printright(self.x + self.w - 2, self.y, themeColor(THEME_INFOFORE), (batt >= 0) and ("%d%%"):format(batt) or "CHRG")
end

--------------------------------------------------------------------------------

StatusBar = class(Widget)

function StatusBar:__init(x, y, w, h, buttons)
   Widget.__init(self, x, y, w, h)
   self:setbuttons(buttons)
end

function StatusBar:setbuttons(buttons)
   self.buttons = buttons or {}
end

--~ function StatusBar:update(delta)
   --~ Widget.update(self, delta)
--~ end

function StatusBar:draw()
   pge.gfx.drawrect(self.x, self.y, self.w, self.h, themeColor(THEME_INFOBACK))

   local x = self.x + 1
   for i,b in ipairs(BUTTONS) do
      if self.buttons[b] then
         x = BUTTONPICS:print(x, self.y, themeColor(THEME_INFOFORE), BUTTONTEXT[b])
         font:activate()
         --x = font:print(x, self.y, themeColor(THEME_INFOFORE), BUTTONTEXT[b])
         x = font:print(x, self.y, themeColor(THEME_INFOFORE), self.buttons[b]) + 16
         --x = x + font:measure(self.buttons[b]) + 16 -- padding between items
      end
   end

    if self.buttons.diskfree then
        font:activate()
        local s = '%d MiB free' % pge.io.msfree()
        font:printright(self.x + self.w - 2, self.y, themeColor(THEME_INFOFORE), s)
    end
end
