--require 'pgex'

print "FFN Browser starting..."

fonts = {
---[[
   'font/Ubuntu-R.ttf', -- normal
   'font/Ubuntu-B.ttf', -- bold
   'font/Ubuntu-I.ttf', -- italic
   'font/Ubuntu-BI.ttf' -- bold+italic
--]]
--[[
   'font/Vera.ttf',     -- normal
   'font/VeraBd.ttf',   -- bold
   'font/VeraIt.ttf',   -- italic
   'font/VeraBI.ttf'    -- bold+italic
--]]
--[[
   'font/bookos.ttf',   -- normal
   'font/bookosb.ttf',  -- bold
   'font/bookosi.ttf',  -- italic
   'font/bookosbi.ttf'  -- bold+italic
--]]
   }

font, fontb, fonti, fontbi = unpack(fonts)

--[[
require "class"
require "extend"
require "filepath"
require "dump"

require "XML"
require "FFN"
--require "HTTP"
require "HTML"

require "screenshot"

--------------------------------------------------------------------------------

require "GUI"
require "View"
require "View_Loading"
require "View_Import"
require "View_MainMenu"
require "View_Library"
require "View_Story"
require "View_Categories"
require "View_Search"
require "View_Feeds"
require "View_Feed"
require "View_Entry"
require "View_Download"
require "View_Options"
require "View_Themes"
require "View_CustomTheme"
require "View_Orient"
require "View_Closing"
]]

--------------------------------------------------------------------------------

require 'pge.http'

function needfile(file, overwrite)
   --return overwrite or not pge.file.exists(file)
   io.write(('checking file "%s"... '):format(file))
   if overwrite then print '(overwrite file)' return true end
   if pge.file.exists(file) then print '(file exists)' return false end
   print '(file not found)'
   return true
end

function getfile(file, url, overwrite)
   if url and needfile(file, overwrite) then
      file = file or pge.dir.basename(url):gsub('[\\/:%*%?"\'<>|]', '_')

      if ap_connect() then
         local dir = pge.dir.dirname(file)
         if dir and dir ~= '' and not pge.dir.mkdirs(dir) then
            screen:showDialog(Dialog.Message("Error creating folder:\n"..dir))
            return
         end
         if pge.file.exists(file) then
            printf('Removing "%s"', file)
            pge.file.remove(file)
         end
         printf('Fetching "%s"->"%s"...', url, file)
         local res, code, msg = pge.net.http.download(url, file)
         if res then
            print('OK', res)
         elseif msg then
            print(('Error %d: %s'):format(code, msg))
            return
         else
            print('Unknown error!', code)
            return
         end
      end
   end
   return pge.file.readfile(file)
end

function getchapter(file, url, overwrite)
   local text = getfile(file, url, overwrite)

   if text then
--[[
      local story_start, story_end
      local l = 0

      for s,e,t in text:gfind('(<[^>]+>)') do
         print(t)
         if t == '<!%-%- start story %-%->' then
            story_start = e + 1
            print(('story starts @ %d'):format(story_start))
         elseif t == '<!%-%- end story %-%->' then
            story_end = s - 1
            print(('story ends @ %d'):format(story_end))
         elseif t:match('<div') then
            print(t)
            l = l + 1
            for k,v in t:gmatch('(%w+)=(%w+)') do
               print(k,v)
               if k == 'id' and v == 'storytext' then
                  l = 1
                  story_start = e + 1
                  print(('story starts @ %d'):format(story_start))
                  break
               end
            end
         elseif t:match('</div') then
            if l > 0 then
               l = l - 1
            end
            if l == 0 then
               story_end = s - 1
               print(('story ends @ %d'):format(story_end))
               break
            end
         end
      end

      if story_start and story_end then
         text = text:sub(story_start or 1, story_end or -1)
      end
--]]
--[ [
      local story = text:match('<!%-%- start story %-%->(.-)<!%-%- end story %-%->')
      if story then
         pge.file.writefile(file, story)
         return story
      end
      local story = text:match('<div id=storytext class=storytext>(.-)</div>')
      if story then
         pge.file.writefile(file, story)
         return story
      end
--[[
      local x = mxml.loadstring(text)
      if x then
         local node = x:find("div","id","storytext")
         error("node.class=="..node:attr("class"))
      else
         --error("XML parse error")
      end
--]]
   end

   return text
end

function addinfo(sid, title, chapters, aid, author)
    if aid then
        local t = db.author:get(aid, {})
        t.name = author or t.author
    end

    if sid then
        local t = db.story:get(sid, {})
        t.title = title or t.title
        t.author = aid or t.author

        if not t.nchapters or (chapters and chapters > t.nchapters) then
            t.new = true -- highlight new stories/chapters
        end

        t.nchapters = chapters or t.nchapters
    end
end

function getfeed(file, url, overwrite)
    local feed = Feed(file, url, overwrite)

    if feed then
        for i, entry in ipairs(feed.entry) do
            addinfo(FFN.storyid(entry.link.alternate), entry.title,
                    FFN.nchapters(entry.summary),
                    FFN.userid(entry.author.uri), entry.author.name)
        end
    end

    return feed
end

--------------------------------------------------------------------------------

offline = false

function ap_connect()
   if not offline and not pge.net.isconnected() then
      if screen:showDialog(Dialog.Network('connect', true)) ~= 0 then
         offline = true
      end
      --offline = not pge.net.connect()
   end
   return pge.net.isconnected()
end

function ap_disconnect()
   pge.net.disconnect()
   offline = false
end

--------------------------------------------------------------------------------

Database = class()

Database._file = {}

function Database:__init(file)
--[[
   local db = sqlite.open(file..'.sqlite')
   if not db:exec('CREATE TABLE IF NOT EXISTS `table`(`key` VARCHAR(100) PRIMARY KEY, `value` TEXT)') then
      print(db:error())
   end
   db:close()
]]
   self:load(file)
end

function Database:load(file)
   file = file or Database._file[self]
   local t = loadfile(file)
   for k, v in pairs(t and t() or {}) do
      self[k] = v
   end
--[[
   local db = sqlite.open(file..'.sqlite')
   local q = db:query('SELECT `key`,`value` FROM `table`')
   if not q then print(db:error()) end
   for r in q:rows() do
      k, v = unpack(r)
      self[k] = loadstring(v)()
   end
   db:close()
]]
   Database._file[self] = file
end

function Database:save()
   printf('Saving "%s"...', Database._file[self])
   local r = pge.file.writefile(Database._file[self], table.dump(self))
--[[
   local db = sqlite.open(Database._file[self]..'.sqlite')
   local r = true
   for k, v in pairs(self) do
      if not db:exec('REPLACE INTO `table`(`key`,`value`) VALUES (:k,:v)', k, table.dump(v)) then
         print(db:error())
         r = false
      end
   end
   db:close()
]]
   print('Done.')
   return r
end

function Database:get(key, def)
   --key = tostring(key)
   if self[key] == nil then
      self[key] = def
   end
   return self[key]
end

function Database:set(key, val)
   --key = tostring(key)
   self[key] = val
end

--------------------------------------------------------------------------------

local _tagrep = {
   ['p'] = '\n\n    ',
   ['/p'] = '',
   ['br'] = '\n',
   ['hr'] = '\n-----\n',
   ['a'] = '',
   ['/a'] = '',

   ['b'] = '\001',
   ['/b'] = '\001',
   ['i'] = '\002',
   ['/i'] = '\002',
   ['u'] = '\003',
   ['/u'] = '\003',
}

function format_tags(tag)
   if tag:match('^<[%!%?]') then
      return ''
   end
   local t = tag:match("<(/?%a%w*)%W")
   if not t then print('t == nil, tag ==', tag) end
   return t and _tagrep[t:lower()] or ''
end

--------------------------------------------------------------------------------

function shelve_story(sid)
   if not table.find(db.library, sid) then
      table.insert(db.library, sid)
      db.library:save()
   end
end

--------------------------------------------------------------------------------

--[[
THEME_UPLOAD_URL = 'http://fanfiction.ath.cx/ffn-browser/theme.php'

function urlencode(str)
   return str:gsub('%W', function(c) return '%%%02X' % c:byte() end)
end

function upload_themes()
   offline = false
   if ap_connect() then
      db.theme:save()

      local ok, result = pge.net.postform(
         THEME_UPLOAD_URL,
         'id=' .. urlencode(pge.utils.psid()) ..
         '&name=' .. urlencode(pge.utils.nickname()) ..
         '&themes=' .. urlencode(dump(db.theme))
         )

      if ok then
         local themes = assert(loadstring(result))()

         if type(themes) == 'table' and #themes > 0 then
            screen:showDialog(Dialog.Message(
               "Uploaded themes:\n   " .. table.concat(themes, '\n   ')
               ))
         else
            local err = type(themes) == 'string' and themes or 'unknown error'
            screen:showDialog(Dialog.Message("Upload error: " .. err))
         end
      end
   end
end
]]

db = {}
db.theme = Database('db/theme.lua')

if #db.theme == 0 then
   setDefaultThemes()
end

local timer = pge.timer.create()

pge.gfx.init()

font   = pge.font.load(fonts[1], 14, PGE_VRAM)
fontb  = pge.font.load(fonts[2], 14, PGE_VRAM)

screen:push_view(LoadingView())
screen:doFrame()

fonti  = pge.font.load(fonts[3], 14)
fontbi = pge.font.load(fonts[4], 14)

print(font:measure(' '))
print(fontb:measure(' '))
print(fonti:measure(' '))
print(fontbi:measure(' '))

print(('Loaded %d fonts in %f seconds'):format(4, timer:peekdelta()))
timer = nil

db.library  = Database('db/library.lua')
db.bookmark = Database('db/bookmark.lua')
db.story    = Database('db/story.lua')
db.author   = Database('db/author.lua')
db.feed     = Database('db/feed.lua')

screen:pop_view()

-- Try to import old data
if pge.file.exists('FFNDATA.DAT') then
   screen:push_view(ImportView())
   screen:doFrame()

   local ffndata = loadfile('FFNDATA.DAT')

   if ffndata then
      ffndata = ffndata()

      if ffndata then
         for k, v in pairs(db) do
            local g = db[k]
            if k == 'theme' then k = 'themes' end
            for kk, vv in pairs(ffndata[k] or {}) do
               g[kk] = vv
            end

            g:save()
         end

         local rslt = screen:showDialog(Dialog.Message(
            'Import complete. Delete FFNDATA.DAT?',
            PGE_UTILS_MSG_DIALOG_YESNO_BUTTONS +
            PGE_UTILS_MSG_DIALOG_DEFAULT_BUTTON_NO))

         if rslt == PGE_UTILS_MSG_DIALOG_RESULT_YES then
            pge.file.remove('FFNDATA.DAT')
         else
            pge.file.rename('FFNDATA.DAT', 'FFNDATA.OLD')
         end
      else
         screen:showDialog(Dialog.Message('Import error:\nNo data found in FFNDATA.DAT.'))
      end
   else
      screen:showDialog(Dialog.Message('Import error:\nCould not load FFNDATA.DAT.'))
   end

   screen:pop_view()
end
