--------------------------------------------------------------------------------

function sort_categories (c1, c2)
    return FFN.category[c1].name:lower() < FFN.category[c2].name:lower()
end

local categories = {}
for c, name in pairs(FFN.category) do
   table.insert(categories, c)
end

table.sort(categories, sort_categories)

local function default_cb(cid)
    offline = false
    screen:push_view(FeedView(getfeed(FFN.list(cid))))
end

CategoriesView = class(View)

function CategoriesView:__init(pid, cb)
    View.__init(self, "Categories", {
        [PGE_CTRL_CANCEL] = "Return",
        [PGE_CTRL_ACCEPT] = pid and "Select" or "Enter",
        [PGE_CTRL_LTRIGGER] = "Prev Letter",
        [PGE_CTRL_RTRIGGER] = "Next Letter" },
        ListBox)

    if pid then
        self.title.text = self.title.text .. ': ' .. FFN.category[pid].name
        self.pid = pid
    end

    self.cb = cb or default_cb
    --~ print(cb, self.cb, self.cb == default_cb)

    self.categories = {}

    for i, c in ipairs(categories) do
        if FFN.category[c].parent == pid then
            table.insert(self.control.items, FFN.category[c].name)
            table.insert(self.categories, c)
        end
    end

   --self.control.numbered = true
end

function CategoriesView:interact()
    View.interact(self)

    if pge.controls.pressed(PGE_CTRL_ACCEPT) then
        if self.pid then
            self.cb(self.categories[self.control.selected])
        else
            screen:push_view(CategoriesView(self.categories[self.control.selected], self.cb))
        end
    elseif pge.controls.pressed(PGE_CTRL_LTRIGGER) then
        local t = self.control.selected
        repeat
            self.control.selected = wrap(self.control.selected - 1, 1, #self.control.items)
        until self.control.selected == t or
            FFN.category[self.categories[self.control.selected]].name:sub(1,1):lower() ~=
            FFN.category[self.categories[t]].name:sub(1,1):lower()
        screen:refresh()
    elseif pge.controls.pressed(PGE_CTRL_RTRIGGER) then
        local t = self.control.selected
        repeat
            self.control.selected = wrap(self.control.selected + 1, 1, #self.control.items)
        until self.control.selected == t or
            FFN.category[self.categories[self.control.selected]].name:sub(1,1):lower() ~=
            FFN.category[self.categories[t]].name:sub(1,1):lower()
        screen:refresh()
    elseif pge.controls.pressed(PGE_CTRL_CANCEL) then
        self:close()
    end
end

