--------------------------------------------------------------------------------

LibraryView = class(View)

function LibraryView:__init()
   View.__init(self, "Library", {
      [PGE_CTRL_CANCEL] = "Return",
      [PGE_CTRL_ACCEPT] = "Read",
      [PGE_CTRL_SQUARE] = "Update",
      [PGE_CTRL_TRIANGLE] = "Remove" },
      ListBox)

   self.control.numbered = true

   self:setItems()
end

function LibraryView:setItems()
   for i, sid in ipairs(db.library) do
      local story = db.story[sid]
      if story then
        --~ print('story=',story)
        --~ print('story.author=',story.author)
        --~ print('db=',db)
        --~ print('db.author=',db.author)
        --~ print('db.author[story.author]=',db.author[story.author])
        --~ print('db.author[story.author].name=',db.author[story.author].name)
         self.control.items[i] =
            (db.story[sid].new and '@@"%s" by %s' or '"%s" by %s'):
            format(story.title, db.author:get(story.author, { name = '?' }).name)
      else
         table.remove(self.control.items, i)
         --self.control.items[i] = "?"
      end
   end

   screen:refresh()
end

--[[
function LibraryView:update(delta)
   for i, sid in subset(
      self.stories, self.control.first_line, self.control.max_lines)
   do
      local story = db.story[sid]
      self.control.items[i] = '%d. "%s" by %s' % {
         i, story.title, db.author[story.author].name
         }
   end

   View.update(self, delta)
end
]]
function LibraryView:interact()
   View.interact(self)

   if pge.controls.pressed(PGE_CTRL_ACCEPT) then

      if db.library[self.control.selected] then

         local sid = db.library[self.control.selected]

         screen:push_view(StoryView(sid))

         table.remove(db.library, self.control.selected)
         table.insert(db.library, 1, sid)

         local item = table.remove(self.control.items, self.control.selected)
         table.insert(self.control.items, 1, item)
         self.control.selected = 1

         db.library:save()

         self:setItems()

      end

   elseif pge.controls.pressed(PGE_CTRL_SQUARE) then

      -- no point getting author feeds more than once...
      local authors = {}
      for i, sid in ipairs(db.library) do
         authors[db.story[sid].author] = true
      end

      for k, v in pairs(authors) do
         local file, url = FFN.author(k)
         getfeed(file, url, true)
      end

      self:setItems()

   elseif pge.controls.pressed(PGE_CTRL_TRIANGLE) then

      if db.library[self.control.selected] then

         local rslt = screen:showDialog(Dialog.Message(
            "Remove this story from your library?", false))

         print(rslt)

         if rslt == true then
            table.remove(self.control.items, self.control.selected)
            print(#db.library)
            table.remove(db.library, self.control.selected)
            print(#db.library)
            db.library:save()
         end

      end

   elseif pge.controls.pressed(PGE_CTRL_CANCEL) then

      self:close()

   end
end
