--------------------------------------------------------------------------------

SearchView = class(View)

function SearchView:__init()
    View.__init(self, "Search", {
          [PGE_CTRL_CANCEL] = "Return",
          [PGE_CTRL_ACCEPT] = "Search/Read",
          [PGE_CTRL_TRIANGLE] = "Details",
          [PGE_CTRL_LTRIGGER+
           PGE_CTRL_RTRIGGER] = "Change",
        }, ListBox)

    self.search = { type = "story", plus_keywords = "", minus_keywords = "", match = "any", categoryid = 0, }
    self.results = {}

    self:setItems()
end

function SearchView:setItems()
    self.control.items = {}

    table.insert(self.control.items, "Search for: " .. self.search.type)

    if self.search.type == 'story' then
        table.insert(self.control.items, "With words: " .. (self.search.plus_keywords or ''))
        table.insert(self.control.items, "Without words: " .. (self.search.minus_keywords or ''))
        table.insert(self.control.items, "In category: " .. FFN.catname(self.search.categoryid))
    else
        table.insert(self.control.items, "Named: " .. (self.search.plus_keywords or ''))
    end

    table.insert(self.control.items, "Search")

    self.searchindex = #self.control.items

    screen:refresh()
end

function SearchView:doSearch()
    offline = false

    screen:dim(true)

    local search = getfile('search.tmp', FFN.search(self.search), true)

    screen:dim(false)

    self:setItems()

    table.insert(self.control.items, '----------')

    if search then
        pge.file.remove('search.tmp')

        self.results = FFN.searchResults(search)

        if self.results then
            print('search OK, '..tostring(#self.results)..' results')
            for i, v in ipairs(self.results) do
                addinfo(v.sid, v.title, v.chapters, v.aid, v.author)

                if v.title then
                    print(v.sid, '"'..v.title..'"', v.chapters, v.aid, v.author)
                    table.insert(self.control.items, '"'..v.title..'" by '..v.author)
                else
                    print(v.aid, v.author)
                    table.insert(self.control.items, v.author..' ('..tostring(v.stories)..' stories)')
                end
            end

            if #self.results == 0 then
                table.insert(self.control.items, '<no results>')
            end

            return true
        end
    end

    table.insert(self.control.items, '<search error>')
end

local function gettext(t, k, m)
    local s = screen:showDialog(Dialog.OSK(m, t[k]))
    if s and s ~= "" then t[k] = s ; return true end
end

function SearchView:interact()
    View.interact(self)

    if pge.controls.pressed(PGE_CTRL_CANCEL) then
        self:close()
    elseif self.control.selected == 1 then
        if pge.controls.pressed(PGE_CTRL_LTRIGGER) or pge.controls.pressed(PGE_CTRL_RTRIGGER) then
            if self.search.type == 'author' then
                self.search.type = 'story'
            else
                self.search.type = 'author'
            end
            self:setItems()
        end
    elseif self.control.selected == self.searchindex then
        if pge.controls.pressed(PGE_CTRL_ACCEPT) then
            self:doSearch()
        end
    elseif self.control.selected > self.searchindex + 1 then
        if pge.controls.pressed(PGE_CTRL_ACCEPT) then
            print(self.searchindex, self.control.selected - self.searchindex - 1, #self.results)

            if self.search.type == 'author' then
                local aid = self.results[self.control.selected - self.searchindex - 1].aid
                offline = false
                screen:push_view(FeedView(getfeed(FFN.author(aid))))
            elseif self.search.type == 'story' then
                local sid = self.results[self.control.selected - self.searchindex - 1].sid
                screen:push_view(StoryView(sid))
            end
        elseif pge.controls.pressed(PGE_CTRL_TRIANGLE) then
            if self.search.type == 'story' then
                local n = self.control.selected - self.searchindex - 1
                local aid, sid = self.results[n].aid, self.results[n].sid
                local feed = getfeed(FFN.author(aid))

                if feed then
                    for i, entry in ipairs(feed.entry) do
                        if FFN.storyid(entry.link.alternate) == sid then
                            screen:push_view(EntryView(entry))
                            break
                        end
                    end
                end
            end
        end
    elseif pge.controls.pressed(PGE_CTRL_ACCEPT) or
           pge.controls.pressed(PGE_CTRL_LTRIGGER) or
           pge.controls.pressed(PGE_CTRL_RTRIGGER)
    then
        if self.search.type == 'author' then
            if self.control.selected == 2 then
                if gettext(self.search, 'plus_keywords', 'Author Name') then
                    self:setItems()
                end
            end
        else
            if self.control.selected == 2 then
                if gettext(self.search, 'plus_keywords', 'With words') then
                    self:setItems()
                end
            elseif self.control.selected == 3 then
                if gettext(self.search, 'minus_keywords', 'Without words') then
                    self:setItems()
                end
            elseif self.control.selected == 4 then
                print'search: get category'
                screen:push_view(CategoriesView(nil,
                    function(cid)
                        self.search.categoryid = cid
                        self:setItems()
                        local view = screen:view()
                        while view ~= self do
                            view:close()
                            view = screen:view()
                        end
                    end))
            end
        end
    end
end

