--------------------------------------------------------------------------------

ThemesView = class(View)

function ThemesView:__init()
   View.__init(self, "Themes",
      { [PGE_CTRL_CANCEL] = "Return",
        [PGE_CTRL_ACCEPT] = "Edit",
        [PGE_CTRL_SQUARE] = "New",
        [PGE_CTRL_TRIANGLE] = "Remove" },
      ListBox)

   self:setItems()

   --self.control.numbered = true
   self.control.selected = getTheme()
end

function ThemesView:setItems()
   for i, theme in ipairs(db.theme) do
      local s = db.theme[i].name

      if i == db.theme.selected then
         s = '@@'..s
      end

      self.control.items[i] = s
   end

   screen:refresh()
end

function ThemesView:update(delta)
   View.update(self, delta)
end

function ThemesView:interact()
   View.interact(self)

   if pge.controls.pressed(PGE_CTRL_ACCEPT) then
      screen:push_view(CustomThemeView(self.control.selected))
   elseif pge.controls.pressed(PGE_CTRL_TRIANGLE) then
      if #db.theme > 1 then
         local rslt = screen:showDialog(Dialog.Message(
            ('Remove theme "%s"?'):format(db.theme[self.control.selected].name),
            PGE_UTILS_MSG_DIALOG_YESNO_BUTTONS +
            PGE_UTILS_MSG_DIALOG_DEFAULT_BUTTON_NO))

         if rslt == PGE_UTILS_MSG_DIALOG_RESULT_YES then
            table.remove(db.theme, self.control.selected)
            table.remove(self.control.items, self.control.selected)

            if self.control.selected > #db.theme then
               self.control.selected = #db.theme
               setTheme(self.control.selected)
            end

            self:setItems()
         end
      end
   elseif pge.controls.pressed(PGE_CTRL_SQUARE) then
      local t = table.copy(theme())
      t.name = "New Theme"
      table.insert(db.theme, t)
      table.insert(self.control.items, t.name)
      self.control.selected = #db.theme

      self:setItems()

      screen:push_view(CustomThemeView(self.control.selected))
   elseif pge.controls.pressed(PGE_CTRL_CANCEL) then
      self:close()
   --elseif pge.controls.pressed(PGE_CTRL_START) then
   --   upload_themes()
   end

   setTheme(self.control.selected)
end

function ThemesView:on_close()
   db.theme:save()
end

